# -*- coding: iso-8859-1 -*-
"""
    MoinMoin - SharedContent macro, based on the FeedReader macro

    @copyright: 2008, 2012, 2013 by Paul Boddie <paul@boddie.org.uk>
    @license: GNU GPL (v2 or later), see COPYING.txt for details.
"""

from DateSupport import getDateTimeFromISO8601, DateTime
from MoinMoin.Page import Page
from MoinRemoteSupport import *
from MoinSupport import parseMacroArguments
from email.utils import parsedate
import xml.dom.pulldom

try:
    from cStringIO import StringIO
except ImportError:
    from StringIO import StringIO

Dependencies = ["time"]

MAX_ENTRIES = 5
ATOM_NS = "http://www.w3.org/2005/Atom"

# Utility functions.

def text(element):
    nodes = []
    for node in element.childNodes:
        if node.nodeType == node.TEXT_NODE:
            nodes.append(node.nodeValue)
    return "".join(nodes)

def unescape(text):
    return text.replace("&lt;", "<").replace("&gt;", ">").replace("&amp;", "&")

def linktext(element, feed_type):
    if feed_type == "rss":
        return text(element)
    else:
        return element.getAttribute("href")

# Error classes.

class FeedError(Exception):
    pass

class FeedMissingError(FeedError):
    pass

class FeedContentTypeError(FeedError):
    pass

# Entry/update classes.

class Update:

    "A feed update entry."

    def __init__(self):
        self.title = None
        self.link = None
        self.content = None
        self.content_type = None
        self.updated = None

    def __cmp__(self, other):
        if self.updated is None and other.updated is not None:
            return 1
        elif self.updated is not None and other.updated is None:
            return -1
        else:
            return cmp(self.updated, other.updated)

# Feed retrieval.

def getUpdates(request, feed_url, max_entries):

    """
    Using the given 'request', retrieve from 'feed_url' up to the given number
    'max_entries' of update entries.

    A tuple of the form ((feed_type, channel_title, channel_link), updates) is
    returned.
    """

    feed_updates = []

    # Obtain the resource, using a cached version if appropriate.

    max_cache_age = int(getattr(request.cfg, "moin_share_max_cache_age", "300"))
    data = getCachedResource(request, feed_url, "MoinShare", "wiki", max_cache_age)
    if not data:
        raise FeedMissingError

    # Interpret the cached feed.

    feed = StringIO(data)
    _url, content_type, _encoding, _metadata = getCachedResourceMetadata(feed)

    if content_type not in ("application/atom+xml", "application/rss+xml", "application/xml"):
        raise FeedContentTypeError

    try:
        # Parse each node from the feed.

        channel_title = channel_link = None

        feed_type = None
        update = None
        in_source = False

        events = xml.dom.pulldom.parse(feed)

        for event, value in events:

            if not in_source and event == xml.dom.pulldom.START_ELEMENT:
                tagname = value.localName

                # Detect the feed type and items.

                if tagname == "feed" and value.namespaceURI == ATOM_NS:
                    feed_type = "atom"

                elif tagname == "rss":
                    feed_type = "rss"

                # Detect items.

                elif feed_type == "rss" and tagname == "item" or \
                    feed_type == "atom" and tagname == "entry":

                    update = Update()

                # Detect source declarations.

                elif feed_type == "atom" and tagname == "source":
                    in_source = True

                # Handle item elements.

                elif tagname == "title":
                    events.expandNode(value)
                    if update:
                        update.title = text(value)
                    else:
                        channel_title = text(value)

                elif tagname == "link":
                    events.expandNode(value)
                    if update:
                        update.link = linktext(value, feed_type)
                    else:
                        channel_link = linktext(value, feed_type)

                elif feed_type == "atom" and tagname == "content":
                    events.expandNode(value)
                    if update:
                        update.content_type = value.getAttribute("type")
                        if update.content_type in ("xhtml", "application/xhtml+xml", "application/xml"):
                            update.content = value.toxml()
                        else:
                            update.content = text(value)

                elif feed_type == "atom" and tagname == "updated" or \
                    feed_type == "rss" and tagname == "pubDate":
                    events.expandNode(value)

                    if update:
                        if feed_type == "atom":
                            value = getDateTimeFromISO8601(text(value))
                        else:
                            value = DateTime(parsedate(text(value)))
                        update.updated = value

            elif event == xml.dom.pulldom.END_ELEMENT:
                tagname = value.localName

                if feed_type == "rss" and tagname == "item" or \
                    feed_type == "atom" and tagname == "entry":

                    feed_updates.append(update)

                    update = None

                elif feed_type == "atom" and tagname == "source":
                    in_source = False

    finally:
        feed.close()

    return (feed_type, channel_title, channel_link), feed_updates

# The macro itself.

def execute(macro, args):
    request = macro.request
    fmt = macro.formatter
    _ = request.getText

    feed_urls = []
    show_content = None
    max_entries = None

    for arg, value in parseMacroArguments(args):
        if arg == "url":
            feed_urls.append(value)
        elif arg == "show":
            show_content = value in ("true", "True", "yes")
        elif arg == "limit":
            try:
                max_entries = int(value)
            except ValueError:
                return fmt.text(_("SharedContent: limit must be set to the maximum number of entries to be shown"))

    if not feed_urls:
        return fmt.text(_("SharedContent: a feed URL must be specified"))

    show_content = show_content or False
    max_entries = max_entries or MAX_ENTRIES

    # Retrieve updates from feeds, classifying them as missing or bad and
    # excluding them if appropriate.

    updates = []
    feeds = []
    missing = []
    bad_content = []

    for feed_url in feed_urls:
        try:
            feed_info, feed_updates = getUpdates(request, feed_url, max_entries)
            updates += feed_updates
            feeds.append((feed_url, feed_info))
        except FeedMissingError:
            missing.append(feed_url)
        except FeedContentTypeError:
            bad_content.append(feed_url)

    output = []
    append = output.append

    # Show the updates.

    if not show_content:
        append(fmt.bullet_list(on=1))

    # NOTE: Permit configurable sorting.

    updates.sort()
    updates.reverse()

    # Truncate the number of updates to the maximum number.

    updates = updates[:max_entries]

    for update in updates:

        # Emit content where appropriate.
        # NOTE: HTML and XHTML should be sanitised.

        if show_content:
            append(fmt.div(on=1, css_class="moinshare-update"))
            append(fmt.div(on=1, css_class="moinshare-content"))
            if update.content:
                if update.content_type in ("html", "text/html"):
                    append(fmt.rawHTML(unescape(update.content)))
                elif update.content_type in ("xhtml", "application/xhtml+xml"):
                    append(fmt.rawHTML(update.content))
                elif update.content_type in ("text", "text/plain"):
                    append(fmt.text(update.content))
            append(fmt.div(on=0))
            append(fmt.div(on=1, css_class="moinshare-date"))
            append(fmt.text(str(update.updated)))
            append(fmt.div(on=0))
            append(fmt.div(on=0))

        # Or emit title and link information for items.

        elif update.title and update.link:
            append(fmt.listitem(on=1, css_class="moinshare-update"))
            append(fmt.url(on=1, href=update.link))
            append(fmt.icon('www'))
            append(fmt.text(" " + update.title))
            append(fmt.url(on=0))
            append(fmt.listitem(on=0))

    if not show_content:
        append(fmt.bullet_list(on=0))

    # Show the feeds.

    for feed_url, (feed_type, channel_title, channel_link) in feeds:
        if channel_title and channel_link:
            append(fmt.paragraph(on=1, css_class="moinshare-feed"))
            append(fmt.url(on=1, href=channel_link))
            append(fmt.text(channel_title))
            append(fmt.url(on=0))
            append(fmt.text(" "))
            append(fmt.url(on=1, href=feed_url))
            append(fmt.icon('rss'))
            append(fmt.url(on=0))
            append(fmt.paragraph(on=0))

    # Show errors.

    for feed_url in missing:
        append(fmt.paragraph(on=1, css_class="moinshare-missing-feed-error"))
        append(fmt.text(_("SharedContent: updates could not be retrieved for %s") % feed_url))
        append(fmt.paragraph(on=0))

    for feed_url in bad_content:
        append(fmt.paragraph(on=1, css_class="moinshare-content-type-feed-error"))
        return fmt.text(_("SharedContent: updates for %s were not provided in Atom or RSS format") % feed_url)
        append(fmt.paragraph(on=0))

    return ''.join(output)

# vim: tabstop=4 expandtab shiftwidth=4
